import {FC} from 'react';
import styled from 'styled-components';
import {formatCreatedAt} from '../../../../utils/presentation/date';

const Primary = styled('div')({
	display: 'grid',
	gridTemplateColumns: 'repeat(2, 1fr)',
	justifyContent: 'space-between',
	alignItems: 'center',
	gap: '0.5rem',
	'@media (min-width: 720px)': {
		gridTemplateColumns: '1fr',
	},
})

const Name = styled('div')({
	fontWeight: 'bolder',
})

const CreatedAt = styled('time')({
	fontSize: '0.75rem',
	textAlign: 'right',
	'@media (min-width: 720px)': {
		textAlign: 'left',
	},
})

const Data = styled('div')({
	marginTop: '0.5rem',
	fontFamily: 'monospace',
	fontSize: '0.75rem',
	height: '3.75em',
	lineHeight: 1.25,
	overflow: 'hidden',
})

type Props = {
	name: string,
	data: string,
	createdAt: Date,
}

const RingtoneCardDisplay: FC<Props> = ({
	name,
	data,
	createdAt: rawCreatedAt,
}) => {
	const createdAt = new Date(rawCreatedAt)
	return (
		<>
			<Primary>
				<Name>
					{name}
				</Name>
				<CreatedAt
					dateTime={createdAt.toISOString()}
				>
					{formatCreatedAt(createdAt)}
				</CreatedAt>
			</Primary>
			<Data>
				{data}
			</Data>
		</>
	)
}

export default RingtoneCardDisplay
