import {ALLOWED_DURATIONS, Duration} from './duration';

export type Note = {
	duration: Duration,
	dotted?: boolean,
	octave?: number,
	pitchClass?: PitchClass,
}

export type PitchClass = 'c' | 'c#' | 'd' | 'd#' | 'e' | 'f' | 'f#' | 'g' | 'g#' | 'a' | 'a#'

export const PITCH_CLASSES = 'c,c#,d,d#,e,f,f#,g,g#,a,a#,b'.split(',') as readonly PitchClass[]

export const stringifyNote = ({ octave: maybeOctave, pitchClass: maybePitchClass, dotted, duration }: Note) => {
	const octave = maybeOctave as number
	const pitchClass = maybePitchClass as PitchClass
	if (isNaN(octave) || !pitchClass || PITCH_CLASSES.indexOf(pitchClass) < 0) {
		return dotted ? `${duration}p.` : `${duration}p`
	}
	return dotted ? `${duration}${pitchClass}${octave}.` : `${duration}${pitchClass}${octave}`
}

export const parseNote = (s: string): Note => {
	const duration = parseInt(s) as Duration
	if (!ALLOWED_DURATIONS.includes(duration)) {
		throw new TypeError('Invalid duration')
	}

	const durationStr = duration.toString()
	const dotted = s.endsWith('.')
	const pitchClass = s.slice(durationStr.length, dotted ? -2 : -1) as PitchClass
	const pitchClassIndex = PITCH_CLASSES.indexOf(pitchClass)
	if (pitchClassIndex !== -1) {
		const octave = parseInt(dotted ? s.slice(-2) : s.slice(-1))
		if (!isNaN(octave)) {
			return {
				duration,
				dotted,
				octave,
				pitchClass,
			}
		}
	}
	return {
		duration,
		dotted,
	}
}
