import {FastifyPluginAsync} from 'fastify'
import {container} from 'tsyringe'
import RingtoneController from '../../../modules/ringtone/controller'

const ringtones: FastifyPluginAsync = async (fastify): Promise<void> => {
	const ringtoneController = container.resolve<RingtoneController>('RingtoneController')
	fastify.route({
		url: '/:id',
		method: 'GET',
		handler: ringtoneController.get,
	})

	fastify.route({
		url: '/',
		method: 'GET',
		handler: ringtoneController.browse,
	})

	fastify.route({
		url: '/',
		method: 'POST',
		handler: ringtoneController.create,
		preValidation: fastify.authenticate,
	})

	fastify.route({
		url: '/:id',
		method: 'PATCH',
		handler: ringtoneController.update,
		preValidation: fastify.authenticate,
	})

	fastify.route({
		url: '/:id/delete',
		method: 'POST',
		handler: ringtoneController.softDelete,
		preValidation: fastify.authenticate,
	})

	fastify.route({
		url: '/:id/delete',
		method: 'DELETE',
		handler: ringtoneController.undoDelete,
		preValidation: fastify.authenticate,
	})

	fastify.route({
		url: '/:id',
		method: 'DELETE',
		handler: ringtoneController.hardDelete,
		preValidation: fastify.authenticate,
	})
}

export default ringtones
